@@ -113,6 +113,9 @@ class GnomeShell:
     def reload_theme(self):
         self._execute_js('const Main = imports.ui.main; Main.loadTheme();')
 
+    def uninstall_extension(self, uuid):
+        pass
+
     @property
     def mode(self):
         return self._proxy.mode
@@ -150,29 +153,42 @@ class GnomeShell34(GnomeShell32):
     def reload_theme(self):
         logging.warning("Reloading Theme Not Supported")
 
+    def uninstall_extension(self, uuid):
+        return self._proxy.proxy.UninstallExtension('(s)', uuid)
+
 class GnomeShell36(GnomeShell34):
 
     def list_extensions(self):
         return self._proxy.proxy_extensions.ListExtensions()
 
+    def uninstall_extension(self, uuid):
+        return self._proxy.proxy_extensions.UninstallExtension('(s)', uuid)
+
+    def install_remote_extension(self, uuid, reply_handler, error_handler, user_data):
+        self._proxy.proxy_extensions.InstallRemoteExtension('(s)', uuid, result_handler=reply_handler, error_handler=error_handler, user_data=user_data)
+
 @gtweak.utils.singleton
 class GnomeShellFactory:
     def __init__(self):
-        proxy = _ShellProxy()
-        settings = GSettingsSetting("org.gnome.shell")
-        v = map(int,proxy.version.split("."))
+        try:
+            proxy = _ShellProxy()
+            settings = GSettingsSetting("org.gnome.shell")
+            v = map(int,proxy.version.split("."))
 
-        if v >= [3,5,0]:
-            self.shell = GnomeShell36(proxy, settings)
-        elif v >= [3,3,2]:
-            self.shell = GnomeShell34(proxy, settings)
-        elif v >= [3,1,4]:
-            self.shell = GnomeShell32(proxy, settings)
-        else:
-            logging.warn("Shell version not supported")
+            if v >= [3,5,0]:
+                self.shell = GnomeShell36(proxy, settings)
+            elif v >= [3,3,2]:
+                self.shell = GnomeShell34(proxy, settings)
+            elif v >= [3,1,4]:
+                self.shell = GnomeShell32(proxy, settings)
+            else:
+                logging.warn("Shell version not supported")
+                self.shell = None
+
+            logging.debug("Shell version: %s", str(v))
+        except:
             self.shell = None
-
-        logging.debug("Shell version: %s", str(v))
+            logging.warn("Shell not installed or running")
 
     def get_shell(self):
         return self.shell
