@@ -20,12 +20,15 @@ import logging
 import tempfile
 import shutil
 import subprocess
+import glob
 
 import gtweak
 from gtweak.gsettings import GSettingsSetting
-from gtweak.gconf import GConfSetting
 
+from gi.repository import GObject
 from gi.repository import GLib
+from gi.repository import Gio
+from gi.repository import Notify
 
 def singleton(cls):
     """
@@ -111,10 +114,30 @@ def execute_subprocess(cmd_then_args, block=True):
         stdout, stderr = p.communicate()
         return stdout, stderr, p.returncode
 
+@singleton
 class AutostartManager:
-    def __init__(self, desktop_filename, autostart_desktop_filename="", exec_cmd="", extra_exec_args=""):
-        self.desktop_filename = desktop_filename
-        self._autostart_desktop_filename = autostart_desktop_filename or desktop_filename
+
+    @staticmethod
+    def get_desktop_files():
+        return [a.get_filename() for a in Gio.app_info_get_all()]
+
+    @staticmethod
+    def get_user_autostart_files():
+        return glob.glob(
+                    os.path.join(
+                        GLib.get_user_config_dir(), "autostart", "*.desktop")) 
+
+    @staticmethod
+    def get_system_autostart_files():
+        f = []
+        for d in GLib.get_system_config_dirs():
+            f.extend( glob.glob(os.path.join(d, "autostart", "*.desktop")) )
+        return f
+
+class AutostartFile:
+    def __init__(self, appinfo, autostart_desktop_filename="", exec_cmd="", extra_exec_args=""):
+        self._desktop_file = appinfo.get_filename()
+        self._autostart_desktop_filename = autostart_desktop_filename or os.path.basename(self._desktop_file)
         self._exec_cmd = exec_cmd
         self._extra_exec_args = " %s\n" % extra_exec_args
 
@@ -125,46 +148,11 @@ class AutostartManager:
             except:
                 logging.critical("Could not create autostart dir: %s" % user_autostart_dir)
 
-        #find the desktop file
-        self._desktop_file = ""
-        for f in self._get_desktop_files():
-            if os.path.exists(f):
-                self._desktop_file = f
         self._user_autostart_file = os.path.join(user_autostart_dir, self._autostart_desktop_filename)
 
         logging.debug("Found desktop file: %s" % self._desktop_file)
         logging.debug("User autostart desktop file: %s" % self._user_autostart_file)
 
-    def _get_system_autostart_files(self):
-        return [
-            os.path.join(d, "autostart", self._autostart_desktop_filename)
-                for d in GLib.get_system_config_dirs()]
-
-    def _get_desktop_files(self):
-        dirs = [gtweak.DATA_DIR, GLib.get_user_data_dir()]
-        dirs.extend(GLib.get_system_data_dirs())
-        return [os.path.join(d, "applications", self.desktop_filename) for d in dirs]
-
-    def get_autostart_condition(self):
-        for f in self._get_system_autostart_files():
-            if os.path.exists(f):
-                with open(f, 'r') as f:
-                    for l in f.readlines():
-                        if l.startswith("AutostartCondition="):
-                            return l.split("=")[-1].strip()
-        return None
-
-    def uses_autostart_condition(self, autostart_type=None):
-        asc = self.get_autostart_condition()
-        try:
-            if autostart_type:
-                return asc.split(" ", 1)[0] == autostart_type
-            else:
-                return asc != None
-        except:
-            logging.warning("Testing for expected AutostartCondition failed: Got (%s)" % asc, exc_info=True)
-            return False
-
     def is_start_at_login_enabled(self):
         if os.path.exists(self._user_autostart_file):
             #prefer user directories first
@@ -174,8 +162,8 @@ class AutostartManager:
             return open(self._user_autostart_file).read().find("X-GNOME-Autostart-enabled=false") == -1
         else:
             #check the system directories
-            for f in self._get_system_autostart_files():
-                if os.path.exists(f):
+            for f in AutostartManager().get_system_autostart_files():
+                if os.path.basename(f) == self._autostart_desktop_filename:
                     return True
         return False
 
@@ -185,26 +173,6 @@ class AutostartManager:
             logging.info("Removing user autostart file %s" % self._user_autostart_file)
             os.remove(self._user_autostart_file)
 
-        asc = self.get_autostart_condition()
-        if asc:
-            #AutostartCondition=GNOME /desktop/gnome/interface/accessibility
-            #AutostartCondition=GNOME3 if-session gnome-fallback
-            #AutostartCondition=GSettings org.gnome.desktop.background show-desktop-icons
-            method, value = asc.split(" ", 1)
-            try:
-                logging.info("Changing AutostartCondition %s -> %s" % (method, update))
-                if method == "GSettings":
-                    schema, key = value.split(" ")
-                    GSettingsSetting(schema).set_boolean(key, update)
-                elif method == "GNOME":
-                    GConfSetting(shema, bool).set_value(update)
-                else:
-                    raise Exception("Method not supported")
-            except:
-                logging.warning("Autostart desktop file unsupported AutostartCondition (%s)" % asc, exc_info=True)
-
-            return
-
         if update:
             if (not self._desktop_file) or (not os.path.exists(self._desktop_file)):
                 logging.critical("Could not find desktop file: %s" % self._desktop_file)
@@ -228,30 +196,130 @@ class AutostartManager:
             old.close()
             new.close()
 
-if __name__ == "__main__":
-    gtweak.DATA_DIR = "/usr/share"
-    gtweak.GSETTINGS_SCHEMA_DIR = "/usr/share/glib-2.0/schemas/"
+class SchemaList:
 
-    logging.basicConfig(format="%(levelname)-8s: %(message)s", level=logging.DEBUG)
+    __list = None
 
-    d = AutostartManager("matlab.desktop")
-    print d.desktop_filename, "uses autostartcondition", d.uses_autostart_condition()
-    print d.desktop_filename, "autostarts", d.is_start_at_login_enabled()
-    d.update_start_at_login(True)
-    print d.desktop_filename, "autostarts", d.is_start_at_login_enabled()
-    d.update_start_at_login(False)
-    print d.desktop_filename, "autostarts", d.is_start_at_login_enabled()
+    def __init__(self):
 
-    d = AutostartManager("orca.desktop", "orca-autostart.desktop")
-    print d.desktop_filename, "uses autostartcondition", d.uses_autostart_condition()
-    print d.desktop_filename, "autostartcondition is:", d.get_autostart_condition()
-    print d.desktop_filename, "uses GSettings autostartcondition", d.uses_autostart_condition("GSettings")
-    print d.desktop_filename, "autostarts", d.is_start_at_login_enabled()
+        if SchemaList.__list == None:
+            SchemaList.__list = []
+            
+    def get(self):
+        return SchemaList.__list
+    
+    def insert(self, key_name, schema_name):
+        v = [key_name, schema_name]
+        SchemaList.__list.append(v)
+    
+    def reset(self):
+        for i in SchemaList.__list:
+            s = Gio.Settings(i[1])
+            s.reset(i[0])
+@singleton
+class DisableExtension(GObject.GObject):
+    
+    __gsignals__ = {
+        "disable-extension": (GObject.SIGNAL_RUN_FIRST, GObject.TYPE_NONE,()),
+    }    
+    
+    def __init__(self):
+        GObject.GObject.__init__(self)
+    
+    def disable(self):
+        self.emit("disable-extension")
 
-    d = AutostartManager("dropbox.desktop")
-    print d.desktop_filename, "uses autostartcondition", d.uses_autostart_condition()
-    print d.desktop_filename, "autostarts", d.is_start_at_login_enabled()
+@singleton
+class XSettingsOverrides:
+
+    VARIANT_TYPES = {
+        'Gtk/ShellShowsAppMenu': GLib.Variant.new_int32,
+        'Gtk/EnablePrimaryPaste': GLib.Variant.new_int32,
+    }
+
+    def __init__(self):
+        self._settings = Gio.Settings('org.gnome.settings-daemon.plugins.xsettings')
+        self._variant = self._settings.get_value("overrides")
+
+    def _dup_variant_as_dict(self):
+        items = {}
+        for k in self._variant.keys():
+            try:
+                #variant override doesnt support .items()
+                v = self._variant[k]
+                items[k] = self.VARIANT_TYPES[k](v)
+            except KeyError:
+                pass
+        return items
+
+    def _dup_variant(self):
+        return GLib.Variant('a{sv}', self._dup_variant_as_dict())
+
+    def _set_override(self, name, v):
+        items = self._dup_variant_as_dict()
+        items[name] = self.VARIANT_TYPES[name](v)
+        n = GLib.Variant('a{sv}', items)
+        self._settings.set_value('overrides', n)
+
+    def _get_override(self, name, default):
+        try:
+            return self._variant[name]
+        except KeyError:
+            return default
+
+    #while I could store meta type information in the VARIANT_TYPES
+    #dict, its easiest to do default value handling and missing value
+    #checks in dedicated functions
+    def set_shell_shows_app_menu(self, v):
+        self._set_override('Gtk/ShellShowsAppMenu', int(v))
+    def get_shell_shows_app_menu(self):
+        return self._get_override('Gtk/ShellShowsAppMenu', True)
+    def set_enable_primary_paste(self, v):
+        self._set_override('Gtk/EnablePrimaryPaste', int(v))
+    def get_enable_primary_paste(self):
+        return self._get_override('Gtk/EnablePrimaryPaste', True)
+
+class Notification:
+    def __init__(self, summary, body):
+        if Notify.is_initted() or Notify.init("GNOME Tweak Tool"):
+            self.notification = Notify.Notification.new(
+                                    summary,
+                                    body,
+                                    'gnome-tweak-tool'
+            )
+            self.notification.set_hint(
+                                "desktop-entry",
+                                GLib.Variant('s', 'gnome-tweak-tool'))
+            self.notification.show()
+        else:
+            raise Exception("Not Supported")
+
+@singleton
+class LogoutNotification:
+    def __init__(self):
+        if Notify.is_initted() or Notify.init("GNOME Tweak Tool"):
+            self.notification = Notify.Notification.new(
+                                "Configuration changes requiere restart",
+                                "Your session needs to be restarted for settings to take effect",
+                                'gnome-tweak-tool')
+            self.notification.add_action(
+                                "restart",
+                                "Restart Session",
+                                self._logout, None, None)
+            self.notification.set_hint(
+                                "desktop-entry",
+                                GLib.Variant('s', 'gnome-tweak-tool'))
+            self.notification.show()
+        else:
+            raise Exception("Not Supported")
+
+    def _logout(self, btn, action, unknown):
+        d = Gio.bus_get_sync(Gio.BusType.SESSION, None)
+        proxy = Gio.DBusProxy.new_sync(
+                       d,Gio.DBusProxyFlags.NONE, None,
+                       'org.gnome.SessionManager', 
+                       '/org/gnome/SessionManager', 
+                       'org.gnome.SessionManager',
+                       None)
+        proxy.Logout('(u)', 0)
 
-    d = AutostartManager("nautilus.desktop", "nautilus-autostart.desktop")
-    print d.desktop_filename, "uses autostartcondition", d.uses_autostart_condition()
-    d.update_start_at_login(False)
